// Name	::	Default MiniGame
// ================
// Made by	::	McTwist
// Date	::	10-08-11
// Info	::	Makes a minigame default and makes it possible for the owner to leave
// License	::	Free to use
// ================

$DefaultMiniGame = 0;

// Creates the default minigame
function serverCmdSetDefaultMiniGame(%client)
{
	if (!%client.isAdmin)
	{
		messageClient(%client, '', "\c3You are not allowed to create a default minigame.");
		return;
	}
	if (isObject($DefaultMiniGame))
	{
		messageClient(%client, '', "\c3Default minigame already set.");
		return;
	}
	%mini = %client.minigame;
	if (!isObject(%mini))
	{
		messageClient(%client, '', "\c3You are not in a minigame.");
		return;
	}
	
	$DefaultMiniGame = %mini;
	// Force non-admins to join
	for (%i = 0; %i < ClientGroup.getCount(); %i++)
	{
		%c = ClientGroup.getObject(%i);
		if (!%c.isAdmin && %c.minigame !$= %mini)
			%mini.addMember(%c);
	}
	%mini.ownerBrickGroup = %client.brickGroup;
	%mini.ownerBL_ID = %client.bl_id;
	%mini.ownerName = %client.name;
	messageAll('', "\c3Default MiniGame set.");
	commandToClient(%client, 'OnDefaultMiniGame', 1);
}

// Removes the default minigame
function serverCmdRemoveDefaultMiniGame(%client)
{
	if (!%client.isAdmin)
	{
		messageClient(%client, '', "\c3You are not allowed to remove a default minigame.");
		return;
	}
	if (!isObject($DefaultMiniGame))
	{
		messageClient(%client, '', "\c3Default minigame is not set.");
		return;
	}
	
	$DefaultMiniGame = 0;
	messageAll('', "\c3Default MiniGame removed.");
	commandToClient(%client, 'OnDefaultMiniGame', 0);
}

if (isPackage(DefaultMiniGame))
	deactivatePackage(DefaultMiniGame);

package DefaultMiniGame
{
	// Join minigame
	function servercmdJoinMiniGame(%client, %mini)
	{
		Parent::serverCmdJoinMiniGame(%client, %mini);
		
		if ($DefaultMiniGame $= %mini && %mini.owner $= %client)
			commandToClient(%client, 'SetRunningMiniGame', 1);
	}
	
	// Leave minigame
	function serverCmdLeaveMiniGame(%client)
	{
		%mini = %client.minigame;
		%a = isObject($DefaultMiniGame) && $DefaultMiniGame $= %mini;
		if (%a)
		{
			// Not admin
			if (!%client.isAdmin)
			{
				messageClient(%client, '', "\c3You are not allowed to leave a default minigame.");
				return;
			}
			// The owner
			if (%mini.owner == %client)
			{
				commandToClient(%client, 'SetRunningMiniGame', 0);
				%owner = %mini.owner;
				%mini.owner = 0;
				%mini.ownerBrickGroup = %client.brickGroup;
				%mini.ownerBL_ID = %client.bl_id;
				%mini.ownerName = %client.name;
			}
			%mini.numMembers++;
		}
		
		Parent::serverCmdLeaveMiniGame(%client);
		
		if (%a)
		{
			if (isObject(%owner))
				%mini.owner = %owner;
			%mini.numMembers--;
		}
	}
	
	// Create minigame
	function serverCmdCreateMiniGame(%client, %name, %a, %b)
	{
		if (isObject($DefaultMiniGame))
		{
			messageClient(%client, '', "\c3Remove the default minigame to make more minigames.");
			return;
		}
		%mini.ownerBrickGroup = %client.brickGroup;
		
		Parent::serverCmdCreateMiniGame(%client, %name, %a, %b);
	}
	
	// Remove minigame
	function serverCmdEndMiniGame(%client)
	{
		%mini = %client.minigame;
		if (%mini.owner !$= %client)
		{
			messageClient(%client, '', "\c3You do not own this minigame.");
			return;
		}
		if ($DefaultMiniGame $= %mini && isObject(%mini))
			serverCmdRemoveDefaultMiniGame(%client);
		
		Parent::serverCmdEndMiniGame(%client);
	}
	
	// Join game
	function GameConnection::onClientEnterGame(%client)
	{
		Parent::onClientEnterGame(%client);
		
		%mini = $DefaultMiniGame;
		if (isObject(%mini))
		{
			commandToClient(%client, 'AddMiniGameLine', 
				%mini.ownerName TAB %mini.ownerBL_ID TAB %mini.title TAB %mini.numMembers, 
				%mini, %mini.colorIdx);
			if (%mini.ownerBrickGroup $= %client.brickGroup)
				%mini.owner = %client;
			serverCmdJoinMiniGame(%client, %mini);
		}
	}
	
	// Leave game
	function GameConnection::onClientLeaveGame(%client)
	{
		%mini = %client.minigame;
		%a = isObject($DefaultMiniGame) && $DefaultMiniGame $= %mini;
		if (%a)
		{
			// The owner
			if (%mini.owner == %client)
			{
				commandToClient(%client, 'SetRunningMiniGame', 0);
				%owner = %mini.owner;
				%mini.owner = 0;
				%mini.ownerBrickGroup = %client.brickGroup;
				%mini.ownerBL_ID = %client.bl_id;
				%mini.ownerName = %client.name;
			}
			%mini.numMembers++;
			%mini.removeMember(%client);
		}
		
		Parent::onClientLeaveGame(%client);
		
		if (%a)
		{
			if (isObject(%owner))
				%mini.owner = %owner;
			%mini.numMembers--;
		}
	}
	
	// Manually pick spawn point
	function GameConnection::spawnPlayer(%client)
	{
		%mini = $DefaultMiniGame;
		%a = isObject(%mini) && %client.minigame $= %mini && !isObject(%mini.owner);
		if (%a)
		{
			%owner = %mini.owner;
			%brickGroup = %client.brickGroup;
			
			%mini.owner = %client;
			// Optional
			%client.brickGroup = %mini.ownerBrickGroup;
		}
		
		Parent::spawnPlayer(%client);
		
		if (%a)
		{
			%mini.owner = %owner;
			// Optional
			%client.brickGroup = %brickGroup;
		}
	}
};
activatePackage(DefaultMiniGame);